/* 
   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifndef __PXE_H__
#define __PXE_H__

/* This is generated by running configure */
#include "niloconfig.h"

/* Define the basic data types */
#include <sys/types.h>

/* PXE structure signatures. We find the pxe structure by the client
 * application by scanning memory between the top of the free base
 * memory, or 0xa0000. If the top of free base memory can't be
 * determined, scan from 0xa0000 to 0x10000. */
#define BC_ROMID_SIG   "$BC$"
#define	UNDI_ROMID_SIG "UNDI"
#define	BUSD_ROMID_SIG "BUSD"

#define	PXE_SIG		"!PXE"
#define	PXENV_SIG	"PXENV+"

#define	BC_ROMID_REV	0x00
#define	UNDI_ROMID_REV	0x00
#define	BUSD_ROMID_REV	0x00

#define	PXE_REV		0x00
#define	PXENV_REV	0x0201


/*	PXE base-code ROM identification structure.
 *
 *	This structure must be located within the memory defined by the
 *	base-code option ROM header during IPL.  UNDI option ROMs that do
 *	not have their own base-code will check the 16bit word at offset
 *	16h in each option ROM header.  In the base-code option ROM this
 *	word is the offset of the start of the t_BC_ROMID structure.
 *
 *	The base-code loader routine is exptected to know the size of the
 *	code and data images that need to be copied to base memory.
 */
typedef struct bc_romid_s {
    u_int8_t signature[4];	/* Structure signature is not NULL */
				/* terminated. */
    u_int8_t structlength;	/* Length of this structure in */
				/* bytes. */
    u_int8_t structcksum;	/* Use to make byte checksum of this */
				/* structure == zero. */
    u_int8_t structrev;		/* Structure format revision number. */
    u_int8_t bc_rev[3];		/* API revision number stored in */
       				/* Intel order. */
       				/* Revision 2.1.0 == 0x00, 0x01, 0x02 */
    u_int16_t bc_loader;	/* Offset of base-code loader routine */
       				/* in the option ROM image. */
    u_int16_t stacksize;        /* Minimum stack segment size, in */
       				/* bytes, needed to load and run the */
       				/* base-code. */
    u_int16_t datasize;         /* Base-code runtime code and data */
    u_int16_t codesize;         /* segment sizes. */
} bc_romid_t;


typedef struct undi_romid_s {
    u_int8_t signature[4];	/* Structure signature is not NULL */
       				/* terminated. */
    u_int8_t structlength;	/* Length of this structure in */
       				/* bytes. */
    u_int8_t structcksum;	/* Use to make byte checksum of this */
				/* structure == zero. */
    u_int8_t structrev;		/* Structure format revision number. */
    u_int8_t undi_rev[3];	/* API revision number stored in */
       				/* Intel order. */
       				/* Revision 2.1.0 == 0x00, 0x01, 0x02 */
    u_int16_t undi_loader;	/* Offset of UNDI loader routine */
				/* in the option ROM image. */
    u_int16_t stacksize;        /* Minimum stack segment size, in */
				/* bytes, needed to load and run the */
				/* UNDI. */
    u_int16_t datasize;         /* UNDI runtime code and data */
    u_int16_t codesize;         /* segment sizes. */
    u_int8_t bustype[4];	/* 'ISAR', 'EISA', 'PCIR', 'PCCR' */
} undi_romid_t;


typedef struct pxe_s {
    u_int8_t signature[4];	/* Structure signature is not NULL */
				/* terminated. FIXME: this was 4 */
    u_int8_t structlength;	/* Length of this structure in */
				/* bytes. */
    u_int8_t structcksum;	/* Use to make byte checksum of this */
				/* structure == zero. */
    u_int8_t structrev;		/* Structure format revision number. */
    u_int8_t reserved1;		/* must be zero */
    undi_romid_t *undi;	        /* Far pointer to UNDI ROMID */
    bc_romid_t *base;	        /* Far pointer to base-code ROMID */
    u_int16_t (*entry16point)(u_int16_t func, void *param);
                                /* 16bit stack segment API entry */
				/* point.  This will be seg:off in */
				/* real mode and sel:off in 16:16 */
				/* protected mode. */
    u_int16_t (*entry32point)(u_int16_t func, void *param);
				/* 32bit stack segment API entry */
				/* point.  This will be sel:off. */
				/* In real mode, sel == 0 */
    u_int16_t (*statuscallout)(u_int16_t param);
				/* Address of DHCP/TFTP status */
				/* callout routine. */
    u_int8_t reserved2;		/* must be zero */
    u_int8_t segdesccnt;	/* Number of segment descriptors in */
				/* this structure. */
    u_int16_t firstselector;	/* First segment descriptor in GDT */
				/* assigned to PXE. */
    newsegdesc_t stack;
    newsegdesc_t undidata;
    newsegdesc_t undicode;
    newsegdesc_t undicodewrite;
    newsegdesc_t bc_data;
    newsegdesc_t bc_code;
    newsegdesc_t bc_codewrite;
} pxe_t;

/* The upper memory address used when scanning for the pxe structure */
#define UPPER_PXE_MEM 0x0A0000
/* The lowest memory address used when scanning for the pxe structure */
#define LOWER_PXE_MEM 0x10000

#define CALL_PXE_API(x, y)      pxe_data_struct->entry32point (x, y)

typedef enum pxe_type {
    PXE,
    PXENV
} pxe_type_t;

pxenv_ret_code_t call_pxe_api (u_int16_t index, void *param);
void *find_pxe_data (pxe_type_t type);

#endif /* __PXE_H__ */











